@@ -208,7 +208,10 @@ IOUSBDevice::init(USBDeviceAddress deviceAddress, UInt32 powerAvailable, UInt8 s
 {
 
     if(!super::init())
+    {
+        USBLog(3,"[%p]::init super->init failed", this);
 	return false;
+    }
 
     // allocate our expansion data
     if (!_expansionData)
@@ -226,7 +229,7 @@ IOUSBDevice::init(USBDeviceAddress deviceAddress, UInt32 powerAvailable, UInt8 s
         USBError(1, "%s[%p]::init - Error allocating getConfigLock", getName(), this);
         return false;
     }
-    
+
     _address = deviceAddress;
     _descriptor.bMaxPacketSize0 = maxPacketSize;
     _busPowerAvailable = powerAvailable;
@@ -531,10 +534,10 @@ IOUSBDevice::ResetDevice()
         return kIOReturnNotResponding;
     }
 
-    _portHasBeenReset = false;
+    retain();
+   _portHasBeenReset = false;
     
     USBLog(3, "+%s[%p] ResetDevice for port %d", getName(), this, _portNumber );
-    retain();
     thread_call_enter( _doPortResetThread );
 
     // Should we do a commandSleep/Wakeup here?
@@ -555,7 +558,7 @@ IOUSBDevice::ResetDevice()
     USBLog(3, "-%s[%p] ResetDevice for port %d", getName(), this, _portNumber );
 
     _resetInProgress = false;
-    
+    release();
     return kIOReturnSuccess;
 }
 
@@ -1566,13 +1569,21 @@ IOUSBDevice::GetStringDescriptor(UInt8 index, char *buf, int maxLen, UInt16 lang
         return kIOReturnSuccess;
     }
     
-    // Make sure that desc[1] == kUSBStringDesc and that the length is even
+    // Make sure that desc[1] == kUSBStringDesc 
     //
-    if ( desc[1] != kUSBStringDesc || ((desc[0] & 1) != 0) )
+    if ( desc[1] != kUSBStringDesc )
     {
-        USBLog(2,"%s[%p]::GetStringDescriptor descriptor is not a string (%d  kUSBStringDesc) or length (%d) is odd", getName(), this, desc[1], desc[0]);
+        USBLog(2,"%s[%p]::GetStringDescriptor descriptor is not a string (%d  kUSBStringDesc)", getName(), this, desc[1] );
         return kIOReturnDeviceError;
     }
+	
+	if ( (desc[0] & 1) != 0) 
+	{
+		// Odd length for the string descriptor!  That is odd.  Truncate it to an even #
+		//
+        USBLog(3,"%s[%p]::GetStringDescriptor descriptor length (%d) is odd, which is illegal", getName(), this, desc[0]);
+		desc[0] &= 0xfe;
+	}
 
     request.wLength = len;
     bzero(desc, len);
@@ -1585,15 +1596,15 @@ IOUSBDevice::GetStringDescriptor(UInt8 index, char *buf, int maxLen, UInt16 lang
         USBLog(2,"%s[%p]::GetStringDescriptor reading entire string returned error (0x%x)",getName(), this, err);
         return err;
     }
-
-    // If the 2nd byte is a string descriptor and the 1st byte is even, then we're OK
-    //
-    if ( (desc[1] != kUSBStringDesc) || ( (desc[0] & 1) != 0 ) )
-    {
-        USBLog(2,"%s[%p]::GetStringDescriptor descriptor is not a string (%d  kUSBStringDesc) or length (%d) is odd", getName(), this, desc[1], desc[0]);
-        return kIOReturnDeviceError;
-    }
     
+	if ( (desc[0] & 1) != 0) 
+	{
+		// Odd length for the string descriptor!  That is odd.  Truncate it to an even #
+		//
+        USBLog(3,"%s[%p]::GetStringDescriptor(2) descriptor length (%d) is odd, which is illegal", getName(), this, desc[0]);
+		desc[0] &= 0xfe;
+	}
+
     USBLog(5, "%s[%p]::GetStringDescriptor Got string descriptor %d, length %d, got %d", getName(), this,
              index, desc[0], request.wLength);
 
@@ -1661,37 +1672,45 @@ IOUSBDevice::message( UInt32 type, IOService * provider,  void * argument )
             // as an error.  However, any client that does not implement the message method will return
             // kIOReturnUnsupported, so we have to treat that as not an error
             //
-            USBLog(5, "%s at %d: Hub device name is %s at %d", getName(), _address, _usbPlaneParent->getName(), _usbPlaneParent->GetAddress());
-            rootHub = OSDynamicCast(IOUSBRootHubDevice, _usbPlaneParent);
-            if ( !rootHub )
+            if ( _usbPlaneParent )
             {
-                // Check to see if our parent is still connected. A kIOReturnSuccess means that
-                // our parent is still connected to its parent.
-                //
-                err = _usbPlaneParent->message(kIOUSBMessageHubIsDeviceConnected, NULL, 0);
-            }
-
-           
-            if ( err == kIOReturnSuccess )
-            {
-                iter = _usbPlaneParent->getClientIterator();
-                while( (client = (IOService *) iter->getNextObject())) 
+                _usbPlaneParent->retain();
+                
+                USBLog(5, "%s at %d: Hub device name is %s at %d", getName(), _address, _usbPlaneParent->getName(), _usbPlaneParent->GetAddress());
+                rootHub = OSDynamicCast(IOUSBRootHubDevice, _usbPlaneParent);
+                if ( !rootHub )
                 {
-                    IOSleep(1);
-                    
-                    err = client->message(kIOUSBMessageHubIsDeviceConnected, this, &_portNumber);
-    
-                    // If we get a kIOReturnUnsupported error, treat it as no error
+                    // Check to see if our parent is still connected. A kIOReturnSuccess means that
+                    // our parent is still connected to its parent.
                     //
-                    if ( err == kIOReturnUnsupported )
-                        err = kIOReturnSuccess;
-                    else
-                    
-                    if ( err != kIOReturnSuccess )
-                        break;
+                    err = _usbPlaneParent->message(kIOUSBMessageHubIsDeviceConnected, NULL, 0);
                 }
-                iter->release();
+            
+                if ( err == kIOReturnSuccess )
+                {
+                    iter = _usbPlaneParent->getClientIterator();
+                    while( (client = (IOService *) iter->getNextObject())) 
+                    {
+                        IOSleep(1);
+                        
+                        err = client->message(kIOUSBMessageHubIsDeviceConnected, this, &_portNumber);
+        
+                        // If we get a kIOReturnUnsupported error, treat it as no error
+                        //
+                        if ( err == kIOReturnUnsupported )
+                            err = kIOReturnSuccess;
+                        else
+                        
+                        if ( err != kIOReturnSuccess )
+                            break;
+                    }
+                    iter->release();
+                }
+                _usbPlaneParent->release();
             }
+            else
+                err = kIOReturnNoDevice;
+            
            break;
             
         case kIOUSBMessagePortHasBeenResumed:
@@ -1925,7 +1944,8 @@ IOUSBDevice::ProcessPortResetEntry(OSObject *target)
     
     if (!me)
         return;
-        
+
+    me->retain();
     me->ProcessPortReset();
     me->release();
 }
@@ -1961,7 +1981,9 @@ IOUSBDevice::ProcessPortReset()
     if ( _usbPlaneParent )
     {
         USBLog(3, "%s[%p] calling messageClients (kIOUSBMessageHubResetPort)", getName(), this);
+        _usbPlaneParent->retain();
         err = _usbPlaneParent->messageClients(kIOUSBMessageHubResetPort, &_portNumber, sizeof(_portNumber));
+        _usbPlaneParent->release();
     }
     
    // Recreate pipe 0 object
@@ -2037,7 +2059,9 @@ IOUSBDevice::ProcessPortReEnumerate(UInt32 options)
     if ( _usbPlaneParent )
     {
         USBLog(3, "%s[%p] calling messageClients (kIOUSBMessageHubReEnumeratePort)", getName(), this);
+        _usbPlaneParent->retain();
         err = _usbPlaneParent->messageClients(kIOUSBMessageHubReEnumeratePort, &params, sizeof(IOUSBHubPortReEnumerateParam));
+        _usbPlaneParent->release();
     }
     
     USBLog(5,"-%s[%p]::ProcessPortReEnumerate",getName(),this); 
@@ -2078,11 +2102,12 @@ IOUSBDevice::ProcessPortSuspend(bool suspend)
     //
     if ( _usbPlaneParent )
     {
+        _usbPlaneParent->retain();
         if ( suspend )
             err = _usbPlaneParent->messageClients(kIOUSBMessageHubSuspendPort, &_portNumber, sizeof(_portNumber));
         else
             err = _usbPlaneParent->messageClients(kIOUSBMessageHubResumePort, &_portNumber, sizeof(_portNumber));
-        
+        _usbPlaneParent->release();
     }
    
      _portSuspendThreadActive = false;
@@ -2144,12 +2169,24 @@ IOUSBDevice::GetDeviceRelease(void)
     return USBToHostWord(_descriptor.bcdDevice); 
 }
 
+UInt16  
+IOUSBDevice::GetbcdUSB(void)
+{ 
+    return USBToHostWord(_descriptor.bcdUSB); 
+}
+
 UInt8  
 IOUSBDevice::GetNumConfigurations(void)
 { 
     return _descriptor.bNumConfigurations; 
 }
 
+UInt8  
+IOUSBDevice::GetProtocol(void)
+{ 
+    return _descriptor.bDeviceProtocol; 
+}
+
 UInt8  
 IOUSBDevice::GetManufacturerStringIndex(void ) 
 { 
